/*
 * ----------------------------------------------------------------------------
 *  (C) Copyright Testing Technologies, 2001-2006.  All Rights Reserved.
 *
 *  All copies of this program, whether in whole or in part, and whether
 *  modified or not, must display this and all other embedded copyright
 *  and ownership notices in full.
 *
 *  See the file COPYRIGHT for details of redistribution and use.
 *
 *  You should have received a copy of the COPYRIGHT file along with
 *  this file; if not, write to the Testing Technologies,
 *  Rosenthaler Str. 13, 10119 Berlin, Germany.
 *
 *  TESTING TECHNOLOGIES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *  SOFTWARE. IN NO EVENT SHALL TESTING TECHNOLOGIES BE LIABLE FOR ANY
 *  SPECIAL, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 *  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 *  THIS SOFTWARE.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 *  EITHER EXPRESSED OR IMPLIED, INCLUDING ANY KIND OF IMPLIED OR
 *  EXPRESSED WARRANTY OF NON-INFRINGEMENT OR THE IMPLIED WARRANTIES
 *  OF MECHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * -----------------------------------------------------------------------------
 *
 *  AUTHOR:      dte
 *  DATE:        Apr 24, 2006
 *
 *  REVISION INFO:
 *    $Revision$ $Date$
 *    $Source$
 *
 * -----------------------------------------------------------------------------
 */


package com.testingtech.ttcn.tri;

import java.io.IOException;

import jpcap.Jpcap;
import jpcap.JpcapWriter;
import jpcap.Packet;

/**
 * The intention of this class is to capture all messages and
 * store it in a capture file.
 * 
 * @author dte
 *
 */
public class JpcapWriterRunnable implements Runnable {
	
	private Jpcap f_jpcap = null;
	
	private String f_capPath = "";
	private String f_capName = "";
	
	private boolean runThread = false;
	
	/**
	 * The constructor of the class.
	 * 
	 * @param jpcap			The Jpcap instance for capturing.
	 */
	public JpcapWriterRunnable(Jpcap jpcap, String capPath, String capName) {
		super();
		f_jpcap = jpcap;
		f_capPath = capPath;
		f_capName = capName;
	}
	
	private synchronized void setRunThread(boolean bv) {
		runThread = bv;
	}
	
	private synchronized boolean getRunThread() {
		return runThread;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		synchronized (this) {
			runThread = true;
			notifyAll();
		}

    	JpcapWriter jw;
		try {
			jw = new JpcapWriter(f_jpcap, f_capPath + validateFileName(f_capName));
		} catch (IOException e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
			setRunThread(false);
			return;
		}

		while (getRunThread()) {
			try {
				Thread.sleep(100);
				Packet packet = f_jpcap.getPacket();

				if (packet == null) {
					continue;
				}
				
				if (jw != null) {
					jw.writeDumpFile(packet);
				}
				
			}
			catch (InterruptedException iex) {
				//ignore
			}
		}
		jw.closeDumpFile();
		synchronized (this) {
			notifyAll();
		}
	}
	
	/**
	 * Cancels the running receiver thread
	 */
	public synchronized void cancel() {
		while (!runThread) {
			try {
				wait();
			} catch (InterruptedException e) {
				//ignore
			}
		}
		synchronized (this) {
			runThread = false;
		}
	}
	
    /**
     * Validates the given name to a valid file name
     * @param name
     * @return
     */
    private String validateFileName(String name) {
    	String v_name = name;
    	
    	try {
			v_name = v_name.replaceAll("\\\\", "_");
			v_name = v_name.replaceAll("/", "_");
			v_name = v_name.replaceAll(":", "_");
			v_name = v_name.replaceAll("\\*", "_");
			v_name = v_name.replaceAll("\\?", "_");
			v_name = v_name.replaceAll("<", "_");
			v_name = v_name.replaceAll(">", "_");
			v_name = v_name.replaceAll("\\|", "_");
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
    	
    	return v_name;
    }
}
